
#declare Radiosity = 1;

global_settings
{
    assumed_gamma 1.0
    #if (Radiosity)
    
        radiosity
        {
          pretrace_start 0.08           // start pretrace at this size
          pretrace_end   0.04           // end pretrace at this size
          count 35                      // higher -> higher quality (1..1600) [35]
          nearest_count 5               // higher -> higher quality (1..10) [5]
          error_bound 1.8               // higher -> smoother, less accurate [1.8]
          recursion_limit 3             // how much interreflections are calculated (1..5+) [3]
          low_error_factor 0.5          // reduce error_bound during last pretrace step
          gray_threshold 0.0            // increase for weakening colors (0..1) [0]
          minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
          brightness 1                  // brightness of radiosity effects (0..1) [1]
    
          adc_bailout 0.01/2
          normal on                   // take surface normals into account [off]
          media on                    // take media into account [off]
          //save_file "file_name"       // save radiosity data
          //load_file "file_name"       // load saved radiosity data                   
          always_sample off           // turn sampling in final trace off [on]
          max_sample 1.0              // maximum brightness of samples
        }
        /**/
        /*
        radiosity
        {
            pretrace_start 0.08
            pretrace_end 0.02
            count 35
            error_bound 0.5
            recursion_limit 1
        }
        */
    #end
}

#default 
{
    texture 
    {
        pigment 
        {
            rgb 0.06125
        }
        #if (Radiosity)
            finish 
            {
                ambient 0.0
                diffuse 0.6
                specular 0.03
            }
        #else
            finish 
            {
                ambient 0.1
                diffuse 0.6
                specular 0.3
            }
        #end
    }
}
// ----------------------------------------
#declare cameraLocation = <-5, 6, -18>;
#declare cameraLookAt = <0, 1.5, 0>;

camera
{
	location cameraLocation
	up y*image_height right x*image_width
	look_at cameraLookAt
	angle 45
}

/*
camera 
{
    right x*image_width/image_height
    location  <0, 0, -5>
    look_at   <0, 0, 0>
}
*/

//  Calculate Percent Macro by [GDS|Entropy] Ian Mcdonald
#macro CalculatePercent(Value, Total)
    ((Value / Total) * 100)
#end
// ---------------------------------------- 
   
#macro BlackHoleWarp(Iterations, Falloff, Strength, Position, Radius)
    #local counter = 0; // set counter to 0
    #while (counter < Iterations) 
   
        warp 
        {
            black_hole Position, Radius
            falloff Falloff
            strength Strength
            turbulence 0.4
        }
   
        #local counter = counter + 1;
    #end
#end

#macro Glow(glowPosition, glowRadius, glowScale, glowColor)
    #local MediaFactor = glowScale + glowRadius;

    #local lightFactor = (1 / 24);
    #local density0 = on;
    #local density1 = on;
    #local density2 = on;
    
    #local enableSurroundingFog = true;
    #local enablePrimaryHalo = true;  // get "FixedSimpleVector internal limit exceeded" error when this = true and coreFuzzCount > 17
    #local enableCoreFuzz = true;     // can also produce this error with various combinations of fuzzcount, primary halo enabled, and then rendering a selection
    #local coreFuzzCount = 17;

    #if (enableSurroundingFog)
    //  Surrounding "Fog"
    light_source 
    {
        glowPosition       
        color rgb glowColor * lightFactor
        fade_distance 4 
        fade_power 2
        looks_like 
        {    
            sphere 
            { 
                <0, 0, 0>, glowRadius * 50
                hollow 
                pigment 
                { 
                    rgbt <0, 0, 0, 1> 
                }
                interior 
                {
                    media 
                    {
                        intervals 2         
                        samples 10, 100          
                        emission glowColor
                        method 2
                        #if (density0) 
                            density
                            {
                                spherical
                                scale 8
                                density_map
                                {                         
                                    [0.0 rgb glowColor * 0.0000]
                                    [0.8 rgb glowColor * 0.00765625]
                                }                        
                            }
                        #end
                    }
     
                }        
                finish
                {
                    reflection 0
                    diffuse 0
                    ambient 0
                    phong 0
                    specular 0
                    brilliance 0                
                }             
            } 
        } 
    }
    #end
   
    light_source 
    {
        glowPosition       
        color rgb glowColor
        looks_like 
        { 
            sphere 
            { 
                <0, 0, 0>, glowRadius
                hollow 
                pigment 
                { 
                    rgbt <0, 0, 0, 1> 
                }
                interior 
                {
                    media 
                    {
                        intervals 2         
                        samples 250, 500          
                        emission glowColor
                        method 2
                        #if (density0) 
                            density
                            {
                                spherical
                                scale 3
                                density_map
                                {                         
                                    [0.0 rgb glowColor * 0.000]
                                    [0.8 rgb glowColor * 0.250]
                                    [0.9 rgb glowColor * 0.750]
                                    [1.0 rgb glowColor * 1.000]                        
                                }                        
                            }
                        #end
                        #if (density1)
                            density
                            { 
                                spherical
                                density_map
                                {
                                    [0.00 color rgb glowColor * 0.000]
                                    [ 0.90
                                        granite
                                        color_map 
                                        {
                                            [0 color rgb glowColor * 0.750]
                                            [1 color rgb glowColor * 0.250]
                                        }
                                        BlackHoleWarp(4, 1, 1, <0,0,0>, 1)
                                    ]
                                    [1.00 color rgb glowColor * 1.500]                        
                                }                                                                      
                            } 
                        #end 
                        #if (density2)
                            density
                            { 
                                spherical
                                density_map
                                {
                                    [0.00 color rgb glowColor * 0.000]
                                    [ 0.90
                                        granite
                                        color_map 
                                        {
                                            [0 color rgb glowColor * 2.750]
                                            [1 color rgb glowColor * 2.250]
                                        }
                                        //BlackHoleWarp(4, 1, 1, <0,0,0>, 1)
                                    ]
                                    [1.00 color rgb glowColor * 2.500]                        
                                }                                                                      
                            } 
                        #end                                       
                    }
                }
                finish
                {
                    reflection 0
                    diffuse 0
                    ambient 0
                    phong 0
                    specular 0
                    brilliance 0                
                }             
            } 
        } 
    }
    
    light_source 
    {
        glowPosition       
        color rgb glowColor * 0.75
        fade_distance 4 
        fade_power 3
        looks_like 
        { 
            sphere 
            { 
                <0, 0, 0>, glowRadius * 50
                hollow 
                pigment 
                { 
                    rgbt <0, 0, 0, 1> 
                }
                interior 
                {
                    media 
                    {
                        intervals 2         
                        samples 100, 500          
                        emission glowColor
                        method 2
                        #if (density0) 
                            density
                            {
                                spherical
                                scale 3
                                density_map
                                {                         
                                    [0.0 rgb glowColor * 0.0000]
                                    [0.8 rgb glowColor * 0.00765625]
                                }                        
                            }
                        #end
                    }
                }
                finish
                {
                    reflection 0
                    diffuse 0
                    ambient 0
                    phong 0
                    specular 0
                    brilliance 0                
                }             
            } 
        } 
    }

#if (enableCoreFuzz)
//  Core "Fuzz" 
#local scaleFactor = 3;
#local rad = 5 + glowRadius * scaleFactor;
    light_source 
    {
        glowPosition       
        color rgb (glowColor * lightFactor)
        fade_distance 4 
        fade_power 2
        looks_like 
        {
            sphere 
            { 
                <0, 0, 0>, rad
                hollow 
                pigment 
                { 
                    rgbt <0, 0, 0, 1> 
                }
                interior 
                {
                    media 
                    {
                        intervals 2         
                        samples 250, 500          
                        emission glowColor
                        method 2                        
                        density
                        {
                            spherical
                            scale scaleFactor
                            density_map
                            {
                                [0.0 rgb glowColor * 0.0000] 
                                     
                                #local j = 1;
                                #local jMax = 255;
                                #while (j < jMax)
                                    
                                    #local percentOfTotal = CalculatePercent(j, jMax);
                                        
                                    #local factor = pow(log(exp(j)), ln(percentOfTotal)) * 0.0001;                                                                               
                                    [j * 0.01 rgb (glowColor * (factor))]
                                       
                                    //  Glow Type 3
                                    //#local factor = pow(j, ln(j)) * 0.0001;                                                                               
                                    //[j * 0.01 rgb glowColor * (factor)]
                                        
                                    #local j = j + 1;
                                #end
                            }                        
                        }
                    }                                       
                }
                finish
                {
                    reflection 0
                    diffuse 0
                    ambient 0
                    phong 0
                    specular 0
                    brilliance 0                
                }
                scale 0.125             
            }   // end sphere
        } 
    }
     
#end

    sphere 
    { 
        glowPosition, glowRadius * 0.0153125//0.00765625
        hollow 
        pigment 
        { 
            color rgb glowColor * 1.5 
        }
        finish
        {
            emission 1
            diffuse 1 
            
            reflection 0
            phong 0
            specular 0
            brilliance 0                                
        }
    }
#end

//Types:
#local Float = 0;
#local Vector = < 0, 0, 0>;
#local Color = color rgb < 0, 0, 0>;

#macro SRand(RS) (rand(RS)*2 - 1) #end
#macro RRand(RS, Min, Max) (rand(RS)*(Max-Min) + Min) #end
#macro Clamp(V, Min, Max) (min(Max, max(Min, V))) #end
#macro Range(V, Rmn, Rmx) (V*(Rmx-Rmn) + Rmn) #end
#macro RClamp(V, Rmn, Rmx, Min, Max) (Clamp(Range(V, Rmn, Rmx), Min, Max)) #end

#local RsA = seed(574647);

#local J = Float;
#local J = 0;
#while(J<52)
    Glow
    (
        <SRand(RsA)*8, rand(RsA)*4, SRand(RsA)*8>,                          //  Location 
        rand(RsA)*1.5,                                                      //  Radius
        7,                                                                  //  Scale
        0.65 * <RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1)> //  Color
    )
	#local J = J + 1;
#end

//                  

box
{
	<-15, 0,-10>,
	< 15, 0, 10>
	pigment
	{
	    checker
	    color rgb 0.05,
	    color rgb 1
	}
	finish
	{
		reflection
		{
		    0.5, 0.75 metallic
		}
		diffuse 0.25
		ambient 0
	}
}

sphere
{
	< 0, 2, 0>, 2
	pigment
	{
	    rgb 1
	}
	finish
	{
		reflection
		{
		    0.5, 0.85
		    metallic
		}
		diffuse 0.15
		ambient 0
	}
}