// Persistence Of Vision Ray Tracer Scene Description File
// -------------------------------------------------------
// File: @(#)glowlits.pov
// Description: Here the glow effects are coupled with light sources.
// Features demonstrated: glow
// Creation Date: $ 27 Sep 2000, 16:08:45 $
// Last modified: $ 14 Jun 2004, 22:36:41 $
// Author: Chris Huff
//
// -w320 -h240 +a0.3

//#version unofficial megapov 1.1;

global_settings {
	assumed_gamma 1.0
}

camera {
	location <-5, 6,-18>
	up y*image_height right x*image_width
	look_at < 0, 1.5, 0>
	angle 45
}
   
// ---------------------------------------- 
   
#macro BlackHoleWarp(Iterations, Falloff, Strength, Position, Radius)
    #local counter = 0; // set counter to 0
    #while (counter < Iterations) 
   
        warp 
        {
            black_hole Position, Radius
            falloff Falloff
            strength Strength
            turbulence 0.4
        }
   
        #local counter = counter + 1;
    #end
#end

#macro Glow(glowPosition, glowRadius, glowColor)
#local coreScaleFactor = 0.030625;   //0.00765625//0.0153125//0.030625
#local lightFactor = (1 / 24);
#local density0 = true;
#local density1 = true;
#local density2 = true;

#local enableSurroundingFog = true;
#local enablePrimaryHalo = true;
#local enableCoreFuzz = true;
#local enableBrightCenter = false;

#if (enableSurroundingFog)
//  Surrounding "Fog"
light_source 
{
    glowPosition       
    color rgb glowColor * lightFactor
    fade_distance 4 
    fade_power 2
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, glowRadius * 50
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 2         
                    //samples 100, 500
                    samples 10, 100          
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            scale 8
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.0000]
                                [0.8 rgb glowColor * 0.00765625]
                            }                        
                        }
                    #end
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

#if (enablePrimaryHalo)
//  Primary "Halo"
light_source 
{
    glowPosition       
    color rgb glowColor * lightFactor
    fade_distance 4 
    fade_power 2
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, glowRadius * 10
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 2         
                    //samples 250, 500          
                    samples 10, 100
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            scale 3
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.000]
                                [0.8 rgb glowColor * 0.250]
                                [0.9 rgb glowColor * 0.750]
                                [1.0 rgb glowColor * 1.000]                        
                            }                        
                        }
                    #end
                    #if (density1)
                        density
                        { 
                            spherical
                            density_map
                            {
                                [0.00 color rgb glowColor * 0.000]
                                [ 0.90
                                    granite
                                    color_map 
                                    {
                                        [0 color rgb glowColor * 0.750]
                                        [1 color rgb glowColor * 0.250]
                                    }
                                    //BlackHoleWarp(4, 1, 1, <0,0,0>, 1)
                                ]
                                [1.00 color rgb glowColor * 1.500]                        
                            }                                                                      
                        } 
                    #end 
                    #if (density2)
                        density
                        { 
                            spherical
                            density_map
                            {
                                [0.00 color rgb glowColor * 0.000]
                                [ 0.90
                                    granite
                                    color_map 
                                    {
                                        [0 color rgb glowColor * 2.750]
                                        [1 color rgb glowColor * 2.250]
                                    }
                                ]
                                [1.00 color rgb glowColor * 2.500]                        
                            }                                                                      
                        } 
                    #end                                       
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

//#debug concat("[", str(numElements,0,0), "] \n")
//#debug concat("[", str(numElementsTotal,0,0), "] \n")
//#local fname = "foo.inc";
//#fopen filehandle fname write
//#write (filehandle, indexToUse, "\n")                       
//#fclose filehandle 

#if (enableCoreFuzz)
//  Core "Fuzz"
#local j = 17;
#while (j > 0)

    light_source 
    {
        glowPosition       
        color rgb (glowColor * lightFactor) / 30
        fade_distance 4 
        fade_power 2
        looks_like 
        {       
            sphere 
            { 
                <0, 0, 0>, (glowRadius / (j + 0.05)) * 3//(glowRadius * (coreScaleFactor * 3))
                hollow 
                pigment 
                { 
                    rgbt <0, 0, 0, 1> 
                }
                interior 
                {
                    media 
                    {
                        intervals 2         
                        //samples 100, 500
                        samples 10, 100          
                        emission glowColor
                        method 2
                        #if (density0) 
                            density
                            {
                                spherical
                                density_map
                                {
                                    /*   
                                    #local i = 0;
                                    #while (i < 2)
                                                             
                                        [(0.1 * i) rgb (glowColor * (exp(i) * 0.5))]
                                        
                                        #local i = i + 1;
                                    #end 
                                    */
                                    [0.0 rgb glowColor * 0.0000]
                                    [0.8 rgb glowColor * j]
                                }                        
                            }
                        #end
                    }
                }
                finish
                {
                    reflection 0
                    diffuse 0
                    ambient 0
                    phong 0
                    specular 0
                    brilliance 0                
                }             
            } 
        } 
    }
    #local j = j - 1;
    #end
     
#end

#if (enableBrightCenter)
//  Bright Center
sphere 
{ 
    glowPosition, glowRadius * coreScaleFactor
    hollow 
    pigment 
    { 
        color rgb glowColor * 1.5 
    }
    finish
    {
        ambient 1
        diffuse 1 
        reflection 0
        phong 0
        specular 0
        brilliance 0                                
    }
}
#end

#end   

// ---------------------------------------- 
  
   
//Types:
#local Float = 0;
#local Vector = < 0, 0, 0>;
#local Color = color rgb < 0, 0, 0>;

#macro SRand(RS) (rand(RS)*2 - 1) #end
#macro RRand(RS, Min, Max) (rand(RS)*(Max-Min) + Min) #end
#macro Clamp(V, Min, Max) (min(Max, max(Min, V))) #end
#macro Range(V, Rmn, Rmx) (V*(Rmx-Rmn) + Rmn) #end
#macro RClamp(V, Rmn, Rmx, Min, Max) (Clamp(Range(V, Rmn, Rmx), Min, Max)) #end

#local RsA = seed(574647);

#local J = Float;
#local J = 0;
#while(J<52)
    Glow(< SRand(RsA)*8, rand(RsA)*4, SRand(RsA)*8>, rand(RsA)*1.5, 0.65*< RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1)>)
    /*
	light_source {
		< SRand(RsA)*8, rand(RsA)*4, SRand(RsA)*8>
		color rgb 0.045*< rand(RsA), rand(RsA), rand(RsA)>
		glow { size rand(RsA)*0.65 type int(rand(RsA))}
		fade_distance 4 fade_power 2
	}
	*/
	#local J = J + 1;
#end

box {
	<-15, 0,-10>, < 15, 0, 10>
	pigment {checker color rgb 0.05, color rgb 1}
	finish {
		reflection {0.5, 0.75 metallic }
		diffuse 0.25 ambient 0
	}
}

sphere {
	< 0, 2, 0>, 2
	pigment {rgb 1}
	finish {
		reflection { 0.5, 0.85 metallic }
		diffuse 0.15 ambient 0
	}
}
